<?php

namespace Oro\Bundle\SearchBundle\Command;

use Oro\Bundle\SearchBundle\Engine\IndexerInterface;
use Symfony\Bundle\FrameworkBundle\Command\ContainerAwareCommand;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;

/**
 * Update and reindex (automatically) fulltext-indexed table(s).
 * Use carefully on large data sets - do not run this task too often.
 */
class ReindexCommand extends ContainerAwareCommand
{
    const COMMAND_NAME = 'oro:search:reindex';

    /**
     * {@inheritdoc}
     */
    protected function configure()
    {
        $this
            ->setName(self::COMMAND_NAME)
            ->addArgument(
                'class',
                InputArgument::OPTIONAL,
                'Full or compact class name of entity which should be reindexed' .
                '(f.e. Oro\Bundle\UserBundle\Entity\User or OroUserBundle:User)'
            )
            ->setDescription('Rebuild search index')
        ;
    }

    /**
     * {@inheritdoc}
     */
    protected function execute(InputInterface $input, OutputInterface $output)
    {
        $class = $input->getArgument('class');

        $this->getSearchIndexer()->reindex($class);

        $message = $class
            ? sprintf('Started reindex task for "%s" entity', $class)
            : 'Started reindex task for all mapped entities'
        ;

        $output->writeln($message);
    }

    /**
     * @return IndexerInterface
     */
    protected function getSearchIndexer()
    {
        return $this->getContainer()->get('oro_search.async.indexer');
    }
}
