<?php

namespace Oro\Bundle\UserBundle\Tests\Functional;

use Oro\Bundle\TestFrameworkBundle\Test\WebTestCase;
use Oro\Bundle\UserBundle\Entity\User;
use Oro\Bundle\UserBundle\Entity\UserManager;

/**
 * @dbIsolation
 */
class ControllersResetTest extends WebTestCase
{
    protected function setUp()
    {
        $this->initClient([], $this->generateBasicAuthHeader());
        $this->client->useHashNavigation(true);
        $this->client->followRedirects();
        $this->loadFixtures([
            'Oro\Bundle\UserBundle\Tests\Functional\DataFixtures\LoadUserData'
        ]);
    }

    public function testSetPasswordAction()
    {
        /** @var User $user */
        $user = $this->getReference('simple_user');
        $oldPassword = $user->getPassword();

        $crawler = $this->client->request(
            'GET',
            $this->getUrl('oro_user_reset_set_password', ['id' => $user->getId(), '_widgetContainer' => 'dialog'])
        );
        $result = $this->client->getResponse();
        $this->assertHtmlResponseStatusCodeEquals($result, 200);
        $content = $result->getContent();
        $this->assertContains('oro.user.suggest_password.label', $content);
        $this->assertContains('oro_set_password_form[password]', $content);

        $form = $crawler->selectButton('Save')->form();

        $form['oro_set_password_form[password]'] = $this->generateRandomString(8) . '1Q';

        $this->client->submit($form);
        $result = $this->client->getResponse();
        $this->assertHtmlResponseStatusCodeEquals($result, 200);
        $this->assertContains('oro.user.change_password.flash.success', $result->getContent());

        $user = $this->getContainer()->get('doctrine')->getRepository('OroUserBundle:User')->find($user->getId());

        $this->assertNotNull($user->getPasswordChangedAt());
        $newPassword = $user->getPassword();
        $this->assertNotEquals($oldPassword, $newPassword);
    }

    public function testSendEmailAction()
    {
        $this->client->request(
            'POST',
            $this->getUrl('oro_user_reset_send_email'),
            [
                'username' => self::USER_NAME,
                'frontend' => 1
            ],
            [],
            $this->generateNoHashNavigationHeader()
        );
        $result = $this->client->getResponse();
        $this->assertHtmlResponseStatusCodeEquals($result, 200);
        $this->assertContains('An email has been sent to', $result->getContent());

        /** @var User $user */
        $user = $this->getContainer()->get('doctrine')->getRepository('OroUserBundle:User')->findOneBy(
            ['username' => self::USER_NAME]
        );
        $this->assertNotNull($user->getPasswordRequestedAt());
    }

    public function testSendForcedResetEmailAction()
    {
        /** @var User $user */
        $user = $this->getReference('simple_user');
        $this->assertEquals(UserManager::STATUS_ACTIVE, $user->getAuthStatus()->getId());

        $crawler = $this->client->request(
            'GET',
            $this->getUrl(
                'oro_user_send_forced_password_reset_email',
                ['id' => $user->getId(), '_widgetContainer' => 'dialog']
            )
        );
        $result = $this->client->getResponse();
        $this->assertHtmlResponseStatusCodeEquals($result, 200);

        $this->assertContains('/user/send-forced-password-reset-email', $result->getContent());

        $form = $crawler->selectButton('Reset')->form();
        $this->client->submit($form);
        $result = $this->client->getResponse();
        $this->assertContains('widget.remove', $result->getContent());

        $user = $this->getContainer()->get('doctrine')->getRepository('OroUserBundle:User')->find($user->getId());
        $this->assertEquals(UserManager::STATUS_EXPIRED, $user->getAuthStatus()->getId());
    }

    public function testMassPasswordResetAction()
    {
        /** @var User $user */
        $user = $this->getReference('simple_user');
        /** @var User $user2 */
        $user2 = $this->getReference('simple_user2');

        $ids = [$user->getId()];
        $crawler = $this->client->request(
            'GET',
            $this->getUrl(
                'oro_user_mass_password_reset',
                [
                    'id' => $user->getId(),
                    'gridName' => 'users-grid',
                    'actionName' => 'reset_password',
                    'values' => $ids
                ]
            )
        );
        $result = $this->client->getResponse();
        $this->assertJsonResponseStatusCodeEquals($result, 200);

        $response = json_decode($result->getContent(), true);

        $this->assertContains(
            [
                'successful' => true,
                'count' => 1,
            ],
            $response
        );

        $repo = $this->getContainer()->get('doctrine')->getRepository('OroUserBundle:User');
        $user = $repo->find($user->getId());
        $user2 = $repo->find($user2->getId());

        $this->assertEquals(UserManager::STATUS_EXPIRED, $user->getAuthStatus()->getId());
        $this->assertEquals(UserManager::STATUS_ACTIVE, $user2->getAuthStatus()->getId());
    }
}
